@@ -162,7 +162,8 @@ class FastbootTarget(Target):
                                                  boot_tags=self.driver.get_boot_tags())
                 self._customize_bootloader(self.proc, boot_cmds)
             self._monitor_boot(self.proc, self.tester_ps1, self.tester_ps1_pattern)
-            if self.config.start_fastboot_command:
+            if self.config.start_fastboot_command and not \
+               self.config.android_adb_over_tcp:
                 self.driver.wait_for_adb()
             self._booted = True
             return self.proc
@@ -172,9 +173,12 @@ class FastbootTarget(Target):
             raise OperationFailed(msg)
 
     def power_off(self, proc):
-        super(FastbootTarget, self).power_off(proc)
-        if self.config.power_off_cmd:
+        if self.config.power_off_cmd != "":
             self.context.run_command(self.config.power_off_cmd)
+        else:
+            proc.send("~$")
+            proc.sendline("off")
+        super(FastbootTarget, self).power_off(proc)
         self.driver.finalize(proc)
 
     @contextlib.contextmanager
@@ -220,7 +224,8 @@ class FastbootTarget(Target):
                 # Connect to serial
                 self.proc = self.driver.connect()
                 # Hard reset the platform
-                if self.config.hard_reset_command:
+                if self.config.hard_reset_command or \
+                   self.config.hard_reset_command == "":
                     self._hard_reboot(self.proc)
                 else:
                     self._soft_reboot(self.proc)
