@@ -36,6 +36,8 @@ from lava_dispatcher.errors import (
     OperationFailed,
 )
 
+from master import MasterCommandRunner
+
 
 class VexpressTarget(BootloaderTarget):
 
@@ -99,6 +101,9 @@ class VexpressTarget(BootloaderTarget):
                     "vexpress_uefi_path, vexpress_uefi_backup_path and "
                     "vexpress_usb_mass_storage_device")
 
+        if self.config.vexpress_sky2_mac:
+            self._boot_tags['{SKY2_MAC}'] = self.config.vexpress_sky2_mac
+
     ##################################################################
     # methods inherited from BootloaderTarget and overriden here
     ##################################################################
@@ -164,6 +169,12 @@ class VexpressTarget(BootloaderTarget):
                                                          firmware, bl0, bl1, bl2, bl31, rootfstype, bootloadertype,
                                                          target_type, qemu_pflash=qemu_pflash)
 
+    def boot_master_image(self):
+        super(VexpressTarget, self).boot_master_image()
+        runner = MasterCommandRunner(self)
+        runner.run("touch /forcefsck")
+        runner.run("sync")
+
     def _load_test_firmware(self):
         with self._mcc_setup() as mount_point:
             self._install_test_firmware(mount_point)
