@@ -19,20 +19,30 @@
 # along
 # with this program; if not, see <http://www.gnu.org/licenses>.
 
-import datetime
 import logging
 import yaml
 import zmq
+import zmq.auth
 
 
 class ZMQPushHandler(logging.Handler):
-    def __init__(self, socket_addr, job_id):
+    def __init__(self, socket_addr, master_cert, slave_cert, job_id):
         super(ZMQPushHandler, self).__init__()
 
         # Create the PUSH socket
         # pylint: disable=no-member
         self.context = zmq.Context()
         self.socket = self.context.socket(zmq.PUSH)
+
+        # Load the certificates (if encryption is on)
+        if master_cert is not None and slave_cert is not None:
+            (client_public, client_private) = zmq.auth.load_certificate(slave_cert)
+            self.socket.curve_publickey = client_public
+            self.socket.curve_secretkey = client_private
+
+            (server_public, _) = zmq.auth.load_certificate(master_cert)
+            self.socket.curve_serverkey = server_public
+
         self.socket.connect(socket_addr)
 
         self.job_id = str(job_id)
@@ -61,8 +71,9 @@ class YAMLLogger(logging.Logger):
         super(YAMLLogger, self).__init__(name)
         self.handler = None
 
-    def addZMQHandler(self, socket_addr, job_id):
-        self.handler = ZMQPushHandler(socket_addr, job_id)
+    def addZMQHandler(self, socket_addr, master_cert, slave_cert, job_id):
+        self.handler = ZMQPushHandler(socket_addr, master_cert,
+                                      slave_cert, job_id)
         self.addHandler(self.handler)
         return self.handler
 
