@@ -24,6 +24,7 @@ import glob
 import yaml
 import uuid
 import json
+import logging
 import unittest
 from lava_dispatcher.pipeline.test.fake_coordinator import TestCoordinator
 from lava_dispatcher.pipeline.test.test_basic import Factory
@@ -49,6 +50,7 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         factory = Factory()
         self.client_job = factory.create_kvm_job('sample_jobs/kvm-multinode-client.yaml')
         self.server_job = factory.create_kvm_job('sample_jobs/kvm-multinode-server.yaml')
+        self.job_id = "100"
         self.coord = TestCoordinator()
 
     def _cleanup(self):
@@ -64,8 +66,8 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         """
         Override the socket calls to simply pass messages directly to the TestCoordinator
         """
-        def __init__(self, coord, parameters):
-            super(TestMultinode.TestClient, self).__init__(parameters)
+        def __init__(self, coord, parameters, job_id):
+            super(TestMultinode.TestClient, self).__init__(parameters, job_id)
             self.coord = coord
             self.debug_setup()
             self.client_name = "fake"
@@ -81,7 +83,6 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
             msg.update(self.base_message)
             return json.dumps(self.coord.dataReceived(msg))
 
-    @unittest.skipIf(len(glob.glob('/sys/block/loop*')) <= 0, "loopback support not found")
     def test_multinode_jobs(self):
         self.assertIsNotNone(self.client_job)
         self.assertIsNotNone(self.server_job)
@@ -90,7 +91,6 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.server_job.validate()
         self.assertEqual(self.server_job.pipeline.errors, [])
 
-    @unittest.skipIf(len(glob.glob('/sys/block/loop*')) <= 0, "loopback support not found")
     def test_protocol(self):
         self.assertEqual(
             ['lava-multinode'],
@@ -144,8 +144,14 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
 
         client_protocol = [protocol for protocol in self.client_job.protocols][0]
         server_protocol = [protocol for protocol in self.server_job.protocols][0]
-        self.assertEqual([client_name for client_name in client_protocol.parameters['protocols'][client_protocol.name]['roles']], ['kvm02', 'kvm01', 'yaml_line'])
-        self.assertEqual([client_name for client_name in server_protocol.parameters['protocols'][server_protocol.name]['roles']], ['kvm02', 'kvm01', 'yaml_line'])
+        self.assertEqual(
+            set([client_name for client_name in
+                 client_protocol.parameters['protocols'][client_protocol.name]['roles']]),
+            {'kvm02', 'kvm01', 'yaml_line'})
+        self.assertEqual(
+            set([client_name for client_name in
+                 server_protocol.parameters['protocols'][server_protocol.name]['roles']]),
+            {'kvm02', 'kvm01', 'yaml_line'})
         self.assertEqual(client_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm01'], 'client')
         self.assertEqual(client_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm02'], 'server')
         self.assertEqual(server_protocol.parameters['protocols'][client_protocol.name]['roles']['kvm01'], 'client')
@@ -177,7 +183,6 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
             self.assertIs(True, protocol.valid)
         self.assertIsNone(self.coord.dataReceived({}))
 
-    @unittest.skipIf(len(glob.glob('/sys/block/loop*')) <= 0, "loopback support not found")
     def test_multinode_description(self):
         self.assertIsNotNone(self.client_job)
         self.client_job.validate()
@@ -254,7 +259,9 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.assertTrue(ret == "completed")
 
     def test_client(self):
-        client = TestMultinode.TestClient(self.coord, self.client_job.parameters)
+        client = TestMultinode.TestClient(self.coord,
+                                          self.client_job.parameters,
+                                          self.job_id)
         client.settings['target'] = 'completed'
         self.coord.expectResponse('wait')
         client.initialise_group()
@@ -275,8 +282,11 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         self.coord.newGroup(2)
         self.coord.addClient("completing")
         self.coord.addClient("completed")
-        client = TestMultinode.TestClient(self.coord, self.client_job.parameters)
-        TestMultinode.TestClient(self.coord, self.server_job.parameters)
+        client = TestMultinode.TestClient(self.coord,
+                                          self.client_job.parameters,
+                                          self.job_id)
+        TestMultinode.TestClient(self.coord, self.server_job.parameters,
+                                 self.job_id)
         self.coord.expectResponse('wait')
         client.initialise_group()
         client.settings['target'] = 'completed'
@@ -299,8 +309,12 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         )
 
     def test_wait(self):
-        client = TestMultinode.TestClient(self.coord, self.client_job.parameters)
-        server = TestMultinode.TestClient(self.coord, self.server_job.parameters)
+        client = TestMultinode.TestClient(self.coord,
+                                          self.client_job.parameters,
+                                          self.job_id)
+        server = TestMultinode.TestClient(self.coord,
+                                          self.server_job.parameters,
+                                          self.job_id)
         client.settings['target'] = 'completed'
         self.coord.expectResponse('wait')
         client.initialise_group()
@@ -331,8 +345,12 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         }))
 
     def test_wait_all(self):
-        client = TestMultinode.TestClient(self.coord, self.client_job.parameters)
-        server = TestMultinode.TestClient(self.coord, self.server_job.parameters)
+        client = TestMultinode.TestClient(self.coord,
+                                          self.client_job.parameters,
+                                          self.job_id)
+        server = TestMultinode.TestClient(self.coord,
+                                          self.server_job.parameters,
+                                          self.job_id)
         client.settings['target'] = 'completed'
         self.coord.expectResponse('wait')
         client.initialise_group()
@@ -357,8 +375,12 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
         }))
 
     def test_wait_all_role(self):
-        client = TestMultinode.TestClient(self.coord, self.client_job.parameters)
-        server = TestMultinode.TestClient(self.coord, self.server_job.parameters)
+        client = TestMultinode.TestClient(self.coord,
+                                          self.client_job.parameters,
+                                          self.job_id)
+        server = TestMultinode.TestClient(self.coord,
+                                          self.server_job.parameters,
+                                          self.job_id)
         client.settings['target'] = 'completed'
         self.coord.expectResponse('wait')
         client.initialise_group()
@@ -490,9 +512,11 @@ class TestMultinode(unittest.TestCase):  # pylint: disable=too-many-public-metho
             mn_protocol.collate(reply, params)
         )
 
-        replaceables = [key for key, value in params['message'].items() if key != 'yaml_line' and value.startswith('$')]
+        replaceables = [key for key, value in params['message'].items()
+                        if key != 'yaml_line' and value.startswith('$')]
         for item in replaceables:
-            data = [val for val in reply['message'].items()][0][1]
+            target_list = [val for val in reply['message'].items()]
+            data = target_list[0][1]
             params['message'][item] = data[item]
 
         self.assertEqual(
@@ -518,6 +542,7 @@ class TestProtocol(unittest.TestCase):  # pylint: disable=too-many-public-method
         """
         Unable to test actually sending messages - need a genuine group with clients and roles
         """
+        self.job_id = "100"
         parameters = {
             'target': 'kvm01',
             'protocols': {
@@ -534,7 +559,9 @@ class TestProtocol(unittest.TestCase):  # pylint: disable=too-many-public-method
             }
         }
         self.coord = TestCoordinator()
-        self.protocol = TestProtocol.FakeProtocol(self.coord, parameters)
+        self.protocol = TestProtocol.FakeProtocol(self.coord, parameters,
+                                                  self.job_id)
+        logging.getLogger('dispatcher').addHandler(logging.NullHandler())
 
     def _wrap_message(self, message, role):
         base_msg = {
@@ -598,17 +625,17 @@ class TestProtocol(unittest.TestCase):  # pylint: disable=too-many-public-method
 
     class FakeProtocol(MultinodeProtocol):
 
-        def __init__(self, fake_coordinator, parameters):
+        def __init__(self, fake_coordinator, parameters, job_id):
             # set the name before passing in the parameters based on that name
             self.name = "fake-multinode"
-            super(TestProtocol.FakeProtocol, self).__init__(parameters)
+            super(TestProtocol.FakeProtocol, self).__init__(parameters, job_id)
             self.sock = TestProtocol.FakeClient(fake_coordinator)
             self.debug_setup()
 
         def _connect(self, delay):
             return True
 
-        def finalise_protocol(self):
+        def finalise_protocol(self, device=None):
             """
             Allow the fake coordinator to finalise the protocol
             """
@@ -673,32 +700,6 @@ class TestProtocol(unittest.TestCase):  # pylint: disable=too-many-public-method
         with self.assertRaises(JobError):
             self.protocol(msg)
 
-    def test_lava_send(self):
-        msg = {
-            'request': 'lava_send',
-            'port': 3,
-            'blocksize': 8,
-            'messageID': 'test-id'
-        }
-        msg.update(self.protocol.base_message)
-        self.coord.expectResponse("ack")
-        self.protocol(msg)
-
-    def test_lava_send_message(self):
-        msg = {
-            'request': 'lava_send',
-            'port': 3,
-            'blocksize': 8,
-            'messageID': 'test-id',
-            'message': {
-                'one': 1,
-                'two': 2
-            }
-        }
-        msg.update(self.protocol.base_message)
-        self.coord.expectResponse("ack")
-        self.protocol(msg)
-
 
 class TestDelayedStart(unittest.TestCase):  # pylint: disable=too-many-public-methods
 
@@ -708,6 +709,7 @@ class TestDelayedStart(unittest.TestCase):  # pylint: disable=too-many-public-me
         """
         Unable to test actually sending messages - need a genuine group with clients and roles
         """
+        job_id = "100"
         client_parameters = {
             'target': 'kvm01',
             'protocols': {
@@ -764,9 +766,15 @@ class TestDelayedStart(unittest.TestCase):  # pylint: disable=too-many-public-me
             }
         }
         self.coord = TestCoordinator()
-        self.client_protocol = TestProtocol.FakeProtocol(self.coord, client_parameters)
-        self.server_protocol = TestProtocol.FakeProtocol(self.coord, server_parameters)
-        self.bad_protocol = TestProtocol.FakeProtocol(self.coord, bad_parameters)
+        self.client_protocol = TestProtocol.FakeProtocol(self.coord,
+                                                         client_parameters,
+                                                         job_id)
+        self.server_protocol = TestProtocol.FakeProtocol(self.coord,
+                                                         server_parameters,
+                                                         job_id)
+        self.bad_protocol = TestProtocol.FakeProtocol(self.coord,
+                                                      bad_parameters,
+                                                      job_id)
 
     def test_lava_start(self):
         self.assertTrue(self.client_protocol.delayed_start)
